.include "1200def.inc"
.CSEG
.ORG 0x00
; ______________________________________________________________________________
;      Autoalarmanlage mit IR-Fernbedienung v1.0  by Th. Gollmer in 2001
;       Brenne sie diese Software in so viele Controller wie sie wollen.
;     Eine komerzielle Nutzung des Programmes bleibt aber mir vorbehalten.
; 
; Quarzfrequenz                 : 4 MHz
; Verwendete Low  Register      : 0
; Verwendete High Register      : 9
; Verwendete IRQ's              : 0
; Anzahl Befehle                : 174
; Letzte nderung               : 18.11.2001
; ------------------------------------------------------------------------------
; Anschludefinitionen
.equ Sensor_Quecksilber_1   = PD0
.equ Sensor_Quecksilber_2   = PD1
.equ Sensor_Quecksilber_3   = PD2
.equ Sensor_Motorhaube      = PD3
.equ Sensor_Tuer            = PD4
.equ Sensor_Kofferraum      = PD5
.equ Sensor_Fernbedienung   = PD6
.equ Eingangsbus            = PIND

.equ LED_Ein_Aus            = PB0
.equ LED_Bewegung           = PB1
.equ LED_Kofferraum         = PB2
.equ LED_Tuer               = PB3
.equ LED_Motorhaube         = PB4
.equ Blinker_rechts         = PB5
.equ Blinker_links          = PB6
.equ Hupe                   = PB7
.equ Ausgangsbus            = PORTB
; ------------------------------------------------------------------------------
; Registerdefinitionen
.def r_Schleife    = r16      ;Register fr Schleifen
.def r_tmp         = r17      ;Register fr Temporrdaten
.def r_Pause1      = r18      ;Register zur Pausenlngensteuerung
.def r_Pause2      = r19      ;Register zur Pausenlngensteuerung
.def r_Pause3      = r20      ;Register zur Pausenlngensteuerung
.def r_IR_IN       = r21      ;Hier werden die empfangenen Daten von
.def r_IR_Hilf     = r22      ;der Fernbedienung eingeschrieben
.def r_Quecksilber = r23      ;Hier wird beim Schrfen der Anlage der Zustand
                              ;der Quecksilberschalter 1-3 gespeichert
.def r_LED_Blink   = r24      ;Hilfsregister zum Blinken
                              ;der Leuchtdioden, falls Alarm ausgelst wurde
; ------------------------------------------------------------------------------
; Konstantendefinitionen:
.equ c_ALL_OFF  = 0b00011111  ;Alle LED's aus, Alle Relais aus
.equ c_IR_Check = 0b11111111  ;r_IR_IN = 255 wenn Signal korrekt
.equ c_IR_False = 0b00000000  ;r_IR_IN = 0   wenn Signal falsch
; ------------------------------------------------------------------------------
; Nach dem Einbau und Anschlu der Alarmanlage wird dieser Programmteil
; angesprungen. Er diehnt zur Einstellung der Data-Direction-Register.
; Weiterhin werden nach dem Anschlu der Betriebsspannung als Test erst die
; Hupe und die Blinker 0,5 sec eingeschaltet,
; danach werden alle LED's 0,5 sec lang Aufleuchten.
; Nach dieser Optischen und Akustischen Mitteilung an den Benutzer
; Geht die Anlage in den "Unscharf Status"
; brigens sollte sich der Kopf beim Anschlu der Betriebsspannung so nahe wie
; mglich an der Hupe befinden.
; Dies erhht den Spafaktor ungemein.
Init:
      ldi   r_tmp,0                ;PD0 - PD6
      out   DDRD,r_tmp             ;als Eingnge
      ldi   r_tmp,255              ;PB0 - PB7
      out   DDRB,r_tmp             ;als Ausgnge
      ldi   r_tmp,255              ;Relais an
      out   Ausgangsbus,r_tmp      ;LED's aus
      rcall Pause_Halbe_Sekunde    ;Pause von 0,5 sec
      ldi   r_tmp,0                ;Relais aus
      out   Ausgangsbus,r_tmp      ;LED's ein
      rcall Pause_Halbe_Sekunde    ;Pause von 0,5 sec
      ldi   r_tmp,c_All_Off        ;LED's & Hupe
      out   Ausgangsbus,r_tmp      ;ausschalten
      rjmp  Unscharf
; ------------------------------------------------------------------------------
; Hier wird ein Alarm ausgegeben. Dazu werden Hupe und Blinker alle 0,5 Sekunden
; im wechsel Ein- und Ausgeschaltet.
; Das ganze in 30 Durchgngen. Das ergibt eine Alarmdauer von ca. 30 Sekunden
; Lnger darf es laut StVZO auch nicht sein.
; Zwischendurch wird immer nach einem Signal von der Ferbedienung
; Ausschau gehalten.
; Kann ja sein das der Besitzer gerade auftaucht und dem Krach oder aber
; dem vermeindlichen Dieb ein Ende bereiten will.
; Whrend des Alarms kann man so auch die Ferbedienung drcken um die
; Anlage zu Entschrfen.
Alarm:
      ldi   r_Schleife,31              ;Schleifenzhler = 31
Start_Alarm:
      sbi   Ausgangsbus,Hupe           ;als erstes die Hupe einschalten
      rcall Pruefe_Fernbedienung       ;Wird die Fernbedienung gedrckt ?
      cpi   r_IR_IN,c_IR_Check         ;Vergleichen ob korrektes Signal empfangen wurde
      breq  Unscharf_schalten          ;Ja, dann Anlage Unscharf schlalten
      rcall Pause_Halbe_Sekunde        ;Halbe Sekunde Hupen
      rcall Pruefe_Fernbedienung       ;Wird die Fernbedienung gedrckt ?
      cpi   r_IR_IN,c_IR_Check         ;Vergleichen ob korrektes Signal empfangen wurde
      breq  Unscharf_schalten          ;Ja, dann Anlage Unscharf schlalten
      cbi   Ausgangsbus,Hupe           ;Hupe ausschalten
      sbi   Ausgangsbus,Blinker_rechts ;Beide Blinker
      sbi   Ausgangsbus,Blinker_links  ;einschalten
      rcall Pruefe_Fernbedienung       ;Wird die Fernbedienung gedrckt ?
      cpi   r_IR_IN,c_IR_Check         ;Vergleichen ob korrektes Signal empfangen wurde
      breq  Unscharf_schalten          ;Ja, dann Anlage Unscharf schlalten
      rcall Pause_Halbe_Sekunde        ;Halbe Sekunde Blinker leuchten lassen
      rcall Pruefe_Fernbedienung       ;Wird die Fernbedienung gedrckt ?
      cpi   r_IR_IN,c_IR_Check         ;Vergleichen ob korrektes Signal empfangen wurde
      breq  Unscharf_schalten          ;Ja, dann Anlage Unscharf schlalten
      cbi   Ausgangsbus,Blinker_rechts ;Beide Blinker wieder
      cbi   Ausgangsbus,Blinker_links  ;ausschalten
      rcall Pruefe_Fernbedienung       ;Wird die Fernbedienung gedrckt ?
      cpi   r_IR_IN,c_IR_Check         ;Vergleichen ob korrektes Signal empfangen wurde
      breq  Unscharf_schalten          ;Ja, dann Anlage Unscharf schlalten
      dec   r_Schleife                 ;Nein, dann Schleifenzhler runterzhlen
      brne  Start_Alarm                ;Wenn noch keine 30 Durchgnge dann zurck
                                       ;Wenn Alarm beendet
      in    r_Quecksilber,Eingangsbus  ;Die Zustnde der Quecksilberschalter neu einlesen
      sbr   r_Quecksilber,248          ;nicht Bewertbare Bits auf 1 setzen
      rjmp  Scharf                     ;Anlage wieder Scharf schalten
; ------------------------------------------------------------------------------
; Diese Routine wird angesprungen, sobald die Anlage Unscharf geschaltet wurde
; Hier wird geprft ob die Fernbedienung gedrckt wird.
; Falls ja wird die Anlage geschrft, wenn nicht wird weiter auf ein Signal
; von der Fernbedienung gewartet.
Unscharf_schalten:
      sbi   Ausgangsbus,LED_Ein_Aus     ;Scharf LED ausschalten
      sbi   Ausgangsbus,Hupe            ;Hupe fr
      rcall Pause_Viertel_Sekunde       ;0,25 sec einschalten
      cbi   Ausgangsbus,Hupe            ;und wieder ausschalten
      sbi   Ausgangsbus,Blinker_rechts  ;Blinker rechts fr 0,5 sec
      rcall Pause_Halbe_Sekunde         ;einschalten
      cbi   Ausgangsbus,Blinker_rechts  ;und wieder ausschalten
      sbi   Ausgangsbus,Blinker_links   ;Jetzt das selbe Spiel
      rcall Pause_Halbe_Sekunde         ;noch mit dem Blinker links
      cbi   Ausgangsbus,Blinker_links   ;und dem Besitzer wurde mitgeteilt, da
                                        ;die Anlage nun Uncharf ist
      in    r_tmp,Ausgangsbus           ;Zustnde der LED's einlesen
      cpi   r_tmp,c_All_OFF             ;Falls noch eine LED leuchtet war ein Alarm
      breq  Unscharf                    ;Wenn nicht dann zu Unscharf springen
      rcall Blink_LED                   ;Falls doch dann LED noch 10 Sekunden blinken lassen
Unscharf:
      ldi   r_tmp,c_All_OFF             ;Relais ausschalten
      out   Ausgangsbus,r_tmp           ;LED's ausschalten
      rcall Pruefe_Fernbedienung        ;Prfen ob die Fernbedienung gedrckt wurde
      cpi   r_IR_IN,c_IR_Check          ;Vergleichen ob korrektes Signal empfangen wurde
      breq  Scharf_schalten             ;Ja, dann Anlage Scharf schalten
      rjmp  Unscharf                    ;Nein, dann zurck
; ------------------------------------------------------------------------------
; Diese Routine wird angesprungen, sobald die Anlage Scharf geschaltet wurde
; Es werden nun stndig alle Sensoren abgefragt, die einen Alarm auslsen
; knnten. Oder die Fernbedienung zum Entschrfen gedrckt wird.
Scharf_schalten:
      cbi   Ausgangsbus,LED_Ein_Aus       ;Scharf LED einschalten
      sbi   Ausgangsbus,Hupe              ;Hupe fr
      rcall Pause_Viertel_Sekunde         ;0,25 sec einschalten
      cbi   Ausgangsbus,Hupe              ;und wieder ausschalten
      sbi   Ausgangsbus,Blinker_rechts    ;Blinker rechts fr 0,5 sec
      rcall Pause_Halbe_Sekunde           ;einschalten
      cbi   Ausgangsbus,Blinker_rechts    ;und wieder ausschalten
      sbi   Ausgangsbus,Blinker_links     ;Jetzt das selbe Spiel
      rcall Pause_Halbe_Sekunde           ;noch mit dem Blinker links
      cbi   Ausgangsbus,Blinker_links     ;und dem Besitzer wurde mitgeteilt, da
                                          ;die Anlage nun Scharf ist
Scharf:
      in    r_Quecksilber,Eingangsbus     ;Die Zustnde der Quecksilberschalter einlesen
      sbr   r_Quecksilber,248             ;nicht Bewertbare Bits auf 1 setzen
Eingaenge_Pruefen:
      sbic  Eingangsbus,Sensor_Motorhaube ;Wird an der Motorhaube rumgefummelt ?
      rjmp  Tuer_Pruefen                  ;Nein, dann weiter
      cbi   Ausgangsbus,LED_Motorhaube    ;Ja, dann LED einschalten
      rjmp  Alarm                         ;und Alarm ausgeben
Tuer_Pruefen:
      sbic  Eingangsbus,Sensor_Tuer       ;Ist ein Dieb zur Tr rein ?
      rjmp  Kofferraum_Pruefen            ;Nein, dann weiter
      cbi   Ausgangsbus,LED_Tuer          ;Ja, dann LED einschalten
      rjmp  Alarm                         ;und Alarm ausgeben
Kofferraum_Pruefen:
      sbic  Eingangsbus,Sensor_Kofferraum ;Klaut einer die Boxen aus dem Kofferraum ?
      rjmp  Bewegung_Pruefen              ;Nein, dann weiter
      cbi   Ausgangsbus,LED_Kofferraum    ;Ja, dann LED einschalten
      rjmp  Alarm                         ;und Alarm ausgeben
Bewegung_Pruefen:
      in    r_tmp,Eingangsbus             ;Zustnde der Quecksilberschalter einlesen
      sbr   r_tmp,248                     ;nicht Bewertbare Bits auf 1 setzen
      cp    r_tmp,r_Quecksilber           ;Hat sich deren Zustand gendert
      breq  Fernbedienung_Pruefen         ;Nein, dann weiter
      cbi   Ausgangsbus,LED_Bewegung      ;Ja, dann LED einschalten
      rjmp  Alarm                         ;und Alarm ausgeben
Fernbedienung_Pruefen:
      rcall Pruefe_Fernbedienung          ;Prfen ob die Fernbedienung gedrckt wurde
      cpi   r_IR_IN,c_IR_Check            ;Vergleichen ob korrektes Signal empfangen wurde
      breq  Unscharf_schalten             ;Ja, dann Anlage Unscharf schlalten
      rjmp  Eingaenge_Pruefen             ;Nein, dann weiter auf Diebe oder den Besitzer warten
; ------------------------------------------------------------------------------
; Diese Unterroutine prft ob die Fernbedienung gedrckt wird.
; Dazu wird im Abstand von 250s Das Signal am Fernbedienungeingang
; abgetastet. Liegt ein IR-Signal an wird nun das Register r_IR_In
; mit dessen Pulscode gefllt.
; Die berprfung ob es sich um ein korrektes Signal handelt
; findet aber erst in den Hauptprogrammteilen statt.
Pruefe_Fernbedienung:
      clr   r_IR_In                              ;Registerinhalt lschen
      in    r_IR_Hilf,Eingangsbus                ;Alle Eingnge ins Register lesen
      bst   r_IR_Hilf,Sensor_Fernbedienung       ;Bit vom Eingang im T-Flag speichern
      bld   r_IR_IN,0                            ;und an Stelle 0 im Empfangsregister speichern
      rcall Pause1000                            ;250 s Synchonisationspause
      in    r_IR_Hilf,Eingangsbus                ;Alle Eingnge ins Register lesen
      bst   r_IR_Hilf,Sensor_Fernbedienung       ;Bit vom Eingang im T-Flag speichern
      bld   r_IR_IN,1                            ;und an Stelle 1 im Empfangsregister speichern
      rcall Pause1000                            ;250 s Synchonisationspause
      in    r_IR_Hilf,Eingangsbus                ;Alle Eingnge ins Register lesen
      bst   r_IR_Hilf,Sensor_Fernbedienung       ;Bit vom Eingang im T-Flag speichern
      bld   r_IR_IN,2                            ;und an Stelle 2 im Empfangsregister speichern
      rcall Pause1000                            ;250 s Synchonisationspause
      in    r_IR_Hilf,Eingangsbus                ;Alle Eingnge ins Register lesen
      bst   r_IR_Hilf,Sensor_Fernbedienung       ;Bit vom Eingang im T-Flag speichern
      bld   r_IR_IN,3                            ;und an Stelle 3 im Empfangsregister speichern
      rcall Pause1000                            ;250 s Synchonisationspause
      in    r_IR_Hilf,Eingangsbus                ;Alle Eingnge ins Register lesen
      bst   r_IR_Hilf,Sensor_Fernbedienung       ;Bit vom Eingang im T-Flag speichern
      bld   r_IR_IN,4                            ;und an Stelle 4 im Empfangsregister speichern
      rcall Pause1000                            ;250 s Synchonisationspause
      in    r_IR_Hilf,Eingangsbus                ;Alle Eingnge ins Register lesen
      bst   r_IR_Hilf,Sensor_Fernbedienung       ;Bit vom Eingang im T-Flag speichern
      bld   r_IR_IN,5                            ;und an Stelle 5 im Empfangsregister speichern
      rcall Pause1000                            ;250 s Synchonisationspause
      in    r_IR_Hilf,Eingangsbus                ;Alle Eingnge ins Register lesen
      bst   r_IR_Hilf,Sensor_Fernbedienung       ;Bit vom Eingang im T-Flag speichern
      bld   r_IR_IN,6                            ;und an Stelle 6 im Empfangsregister speichern
      rcall Pause1000                            ;250 s Synchonisationspause
      in    r_IR_Hilf,Eingangsbus                ;Alle Eingnge ins Register lesen
      bst   r_IR_Hilf,Sensor_Fernbedienung       ;Bit vom Eingang im T-Flag speichern
      bld   r_IR_IN,7                            ;und an Stelle 7 im Empfangsregister speichern
      cpi   r_IR_IN,0b01010101                   ;wenn korrektes
      breq  IR_Korrekt                           ;Pulsweitenmoduliertes
      cpi   r_IR_IN,0b10101010                   ;Signal empfangen wurde
      breq  IR_Korrekt                           ;Sprung zu IR_Korrekt
      ldi   r_IR_IN,C_IR_False                   ;Wenn Signal fasch Register mit Code fr falsch fllen
      ret                                        ;Ende Unterroutine
IR_Korrekt:
      ldi   r_IR_IN,C_IR_Check                   ;Richtiges IR_Signal empfangen
      ret                                        ;Byte mit code fr richtig fllen und Ende Unterroutine
; ------------------------------------------------------------------------------
; Diese Unterroutine generiert eine Pause von 250s=1000 Taktzyklen
Pause1000:
      ldi   r_Pause1,72
      ldi   r_Pause2,2	
Start_Pause_1000:
      dec   r_Pause1
      brne  Start_Pause_1000
      dec   r_Pause2
      brne  Start_Pause_1000
      nop
      ret
; ------------------------------------------------------------------------------
; Diese Unterroutine generiert eine Pause von 0.5 sec
Pause_Halbe_Sekunde:
      ldi   r_Pause1,200
      ldi   r_Pause2,38
      ldi   r_Pause3,11
Start_Pause_Halb:
      dec   r_Pause1
      brne  Start_Pause_Halb
      dec   r_Pause2
      brne  Start_Pause_Halb
      dec   r_Pause3
      brne  Start_Pause_Halb
      ret
; ------------------------------------------------------------------------------
; Diese Unterroutine generiert eine Pause von 0.25 sec
Pause_Viertel_Sekunde:
      ldi   r_Pause1,200
      ldi   r_Pause2,19
      ldi   r_Pause3,6
Start_Pause_Viertel:
      dec   r_Pause1
      brne  Start_Pause_Viertel
      dec   r_Pause2
      brne  Start_Pause_Viertel
      dec   r_Pause3
      brne  Start_Pause_Viertel
      ret
; ------------------------------------------------------------------------------
; Diese Unterroutine lt, falls ein Alarm war beim Enschrfen der Anlage
; die entsprechende LED fr 10 Sekunden blinken, damit der Benutzer wei
; welcher Sensor whrend seiner Abwesenheit einen Alarm ausgelst hat.
; Die LED leuchtet 0,25 sec danach 0,5 nicht.
; Das ganze in 14 durchgngen
; Macht zusammen ca. 10 Sekunden
Blink_LED:
      in    r_LED_Blink,Ausgangsbus    ;momentanen Zustand der LED's ins Register
      ldi   r_tmp,c_All_OFF            ;So sind alle LED's aus
      ldi   r_Schleife,15              ;Schleifenzhler = 15
Start_Blinken:
      out   Ausgangsbus,r_tmp          ;LED's ausschalten
      rcall Pause_Halbe_Sekunde        ;Halbe Sekunde aus lassen
      out   Ausgangsbus,r_LED_Blink    ;LED einschalten
      rcall Pause_Viertel_Sekunde      ;Viertel Sekunde leuchten lassen
      dec   r_Schleife                 ;Durchgnge runterzhlen
      brne  Start_Blinken              ;Noch nicht fertig, dann zurck
      ret                              ;LED hat 10 Sekunden geblinkt
                                       ;Ende Unterroutine
; ______________________________________________________________________________
;      Autoalarmanlage mit IR-Fernbedienung v1.0  by Th. Gollmer in 2001
;       Brenne sie diese Software in so viele Controller wie sie wollen.
;     Eine komerzielle Nutzung des Programmes bleibt aber mir vorbehalten.
; 

